@@ -21,7 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h> // mkstmp()
@@ -615,6 +615,7 @@ static void spl_text_t_from_tracks(text_t** p,
     if(f != NULL) {
       append_text_t(&c, f);
       LIBMTP_PLST_DEBUG("track %d = %s (%u)\n", i+1, f, tracks[i]);
+      free(f);
     }
     else
       LIBMTP_ERROR("failed to find filepath for track=%d\n", tracks[i]);
